/*  Copyright (C) 2010 Faculdade de Ciências e Tecnologia 
 *                     Universidade Nova de Lisboa
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include <stdint.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>

#include "../common/event.h"
#include "../common/xalloc.h"
#include "../common/buffer.h"

#define BUFFER_SIZE 1024

char *get_filename(char *base, int ti, int tf);

struct buffer out_buffer;
int out;

int
flush_f(void *data, size_t len, void *user_data)
{
	write(out,data,len);
        return 0;
}

int
handle_event(struct event *e, int n_transactions, int *trans_count,
	     char *filebase, int *commits, int *aborts)
{
	char *oraw;
	

#if 0
	event_debug_print(e);
#endif

	if (n_transactions > 0)
	{
		oraw=event_get(e);
		buffer_write(&out_buffer,oraw,EVENT_SIZE);
		free(oraw);
	}

	if (e->id == TX_COMMIT
	    || e->id == TX_ABORT)
	{
		if (e->id == TX_COMMIT)
			(*commits)++;
		else if (e->id == TX_ABORT)
			(*aborts)++;

		if (n_transactions <= 0)
			return 0;
		
		(*trans_count)++;
		
		if (!((*trans_count)%n_transactions))
		{
			buffer_flush(&out_buffer);
			close(out);
			
			out=open(get_filename(filebase,
					      *trans_count+1,
					      *trans_count
					      +n_transactions),
				 O_WRONLY|O_CREAT,0700);
			
			if (out < 0)
			{
				perror("open()");
				return -1;
			}
		}
	}
	
	return 0;
}

// pre: base.ends(".bin")
char *
get_filename(char *base, int ti, int tf)
{
	static char filename[1024];
	strcpy(filename,base);
	
	filename[strlen(filename)-4]='\0';
	
	sprintf(filename,"%s-%d-%d.bin",filename,ti,tf);
	
	return filename;
}

int
main(int argc, char **argv)
{
	struct event *e=event_new();
	int in;
	char *raw;
	int r;
	int commits=0;
	int aborts=0;
	int n_transactions=-1;
	int trans_count=0;
	
	if (argc < 2)
	{
		fprintf(stderr,"Usage: %s <trace_file> [<n_transactions>]\n",
			argv[0]);
		fprintf(stderr,"eg: %s foo.bin 1000 2000 2500 3000\n",
			argv[0]);
		fprintf(stderr,"If only the trace_file is passed the"
			" program only outputs some stats\n");
		
		return -1;
	}
	
	if (argc == 3)
		n_transactions=atoi(argv[2]);
	
	raw=xmalloc(EVENT_SIZE*BUFFER_SIZE);
	
	in=open(argv[1],O_RDONLY);
	
	if (in < 0)
	{
		perror("open()");
		return -1;
	}
	
	if (n_transactions > 0)
	{
		out=open(get_filename(argv[1],1,n_transactions),
			 O_WRONLY|O_CREAT,0700);

		buffer_init(&out_buffer,1024*EVENT_SIZE,NULL,flush_f);

		if (out < 0)
		{
			perror("open()");
			return -1;
		}
	}
	
	while ((r=read(in,raw,EVENT_SIZE*BUFFER_SIZE)))
	{
		int events=r/EVENT_SIZE;
		
		if (r%EVENT_SIZE)
		{
			fprintf(stderr,"file corrupted\n");
			return -1;
		}
		
		for (int i=0; i < events; i++)
		{
			event_set(e,raw+EVENT_SIZE*i);
			if (handle_event(e,n_transactions,&trans_count,
					 argv[1],&commits,&aborts))
				return -1;
		}
	}

	if (n_transactions > 0)
		buffer_free(&out_buffer);
	
	event_free(e);
	free(raw);
	close(in);
	close(out);
	
	printf("Total transactions: %d\n",commits+aborts);
	printf("Total commit      : %d\n",commits);
	printf("Total abort       : %d\n",aborts);
	
	return 0;
}
